@@ -562,13 +562,16 @@ def _check_dref(
             isinstance(v0, dict)
             and (
                 isinstance(v0.get('data'), (np.ndarray, list, tuple))
-                or isinstance(v0.get('size'), int)
+                or isinstance(v0.get('size'), (int, np.int_))
             )
         )
         if not c0:
             msg = "v0 must be a dict with either 'data' or 'size'"
             raise Exception(msg)
 
+        if isinstance(v0.get('size'), (int, np.int_)):
+            dref[k0]['size'] = int(v0['size'])
+
         dref2[key] = dict(dref[k0])
 
     # -----------------------
@@ -779,7 +782,9 @@ def _check_data_ref(k0=None, ddata=None, dref0=None, dref_add=None):
     # length mismatch
     elif len(ddata[k0]['shape']) != len(ddata[k0]['ref']):
         msg = (
-            "Mismatching len(ref) and len(shape) for ddata['{k0}']"
+            f"Mismatching len(ref) and len(shape) for ddata['{k0}']\n"
+            f"\t- ref = {ddata[k0]['ref']}\n"
+            f"\t- shape = {ddata[k0]['shape']}\n"
         )
         raise Exception(msg)
 
